# -*- encoding:utf-8 -*-
import os,stat
import sys
import re
import shutil
import random
import string
from werkzeug import script
from flaskext.actions import utils

def preprocess_file(path, **values):
    f = open(path, 'r')
    text = f.read()
    f.close()

    # Save configuration file back
    f = open(path, 'w')
    f.write(text % values)
    f.close()

def startproject(proj_name=''):
    """
    Start new flask project
    """
    if not proj_name:
        sys.stderr.write("proj_name required.\n")
        sys.exit(1)
    if not re.search(r'^[a-zA-Z][a-zA-Z0-9\-]*$', proj_name):
        # If it's not a valid directory name.
        # Provide a smart error message, depending on the error.
        if not re.search(r'^[a-zA-Z]', proj_name):
          message = 'make sure the name begins with a letter'
        else:
          message = 'use only numbers, letters and dashes'
        sys.stderr.write("%r is not a valid project name. Please %s.\n" %
                         (proj_name, message))
        sys.exit(1)
    if os.path.exists(proj_name):
        sys.stderr.write('Folder with name "%s" already exists' % proj_name)
        sys.exit(1)
    from flaskext import actions
    source = os.path.join(actions.__path__[0], 'project_template')
    shutil.copytree(source, proj_name)
    path_join = lambda path: os.path.join(proj_name, path)

    # Rename .py_tmpl to .py
    for fn in os.listdir(proj_name):
        if fn.endswith('.py_tmpl'):
            os.rename(path_join(fn), path_join(fn.replace('_tmpl', '')))

    # Update configuration file
    key = utils.generate_secret_key()
    preprocess_file(path_join('manage.py'), project=proj_name)
    preprocess_file(path_join('__init__.py'), project=proj_name)
    preprocess_file(path_join('settings.py'), project=proj_name,secretkey=key)
    sub_proj_dir = path_join(proj_name)
    os.mkdir(sub_proj_dir)
    shutil.move(path_join('__init__.py'),sub_proj_dir)
    shutil.move(path_join('views'),sub_proj_dir)
    os.chmod(path_join('manage.py'),
             os.stat(path_join('manage.py')).st_mode |
             stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)


script.run(dict(
        startproject = startproject,
        ), '', sys.argv[1:])



# -*- encoding:utf-8 -*-
import string
import random
KEY_CHARS = string.letters + string.digits + string.punctuation

def generate_secret_key(length=32):
    return  repr(''.join(random.choice(KEY_CHARS) for _ in xrange(length)))




# -*- coding: utf-8 -*-
"""
    flaskext.actions
    ~~~~~~~~~~~~~~~~~

    Description of the module goes here...

    :copyright: (c) 2010 by Young King.
    :license: BSD, see LICENSE for more details.
"""
import sys,os
from werkzeug import script
import utils

def runfcgi(application, before_daemon=None):
    def action( hostname            =   ('h', ''),
                port                =   ('p', 3001),
                protocol            =   'scgi',
                method              =   'threaded',
                children            =   20,
                daemonize           =   False,
                pidfile             =   '/var/run/flask.pid',
                workdir             =   '.',
                outlog              =   '/dev/null',
                errlog              =   '/dev/null',
                umask               =   022,
        ):
        """run application use flup
        you can choose these arguments:
        protocol :   scgi, fcgi or ajp
        method   :   threaded or fork
        children :   number of threads or processes"""
        if protocol not in ('scgi', 'fcgi', 'ajp'):
            sys.exit('invalid protocol: %s'%protocol)
        flup_module = 'server.%s'%protocol

        kw = {'maxSpare': children, 'minSpare': children}

        if protocol == 'scgi':
            kw['scriptName'] = '/'

        if method == 'threaded':
            kw['maxThreads'] = children
        elif method == 'fork':
            flup_module += '_fork'
            kw['maxChildren'] = children
        else:
            sys.exit('invalid method: %s'%method)

        try:
            WSGIServer = getattr(__import__('flup.' + flup_module, '', '', flup_module), 'WSGIServer')
        except ImportError:
            print "You need to install flup"
            sys.exit()

        if os.name!='posix' and outlog=='/dev/null':
            outlog = None
        if os.name!='posix' and errlog=='/dev/null':
            errlog = None

        if daemonize:
            from .daemonize import become_daemon
            become_daemon(workdir, outlog, errlog, umask)

        if daemonize and pidfile and os.name=='posix':
            try:
                fp = open(pidfile, 'w')
                fp.write("%d\n" % os.getpid())
                fp.close()
            except:
                pass

        if before_daemon is not None:
            before_daemon()

        WSGIServer(application, bindAddress=(hostname, port), **kw).run()

    return action

def run_twisted_server(app):
    def action(port=('p', 8000), reactor_type=('r', 0)):
        """run application use twisted http server
        :param reactor_type: [default 0]
                            0       default reactor
                            1       epoll   reactor
                            2       poll    reactor
                            3       kqueue  reactor
                            4       iocp    reactor
                            other   select  reactor
        """
        try:
            if reactor_type ==0:
                pass
            elif reactor_type == 1:
                from twisted.internet import epollreactor
                epollreactor.install()
            elif reactor_type == 2:
                from twisted.internet import pollreactor
                pollreactor.install()
            elif reactor_type == 3:
                from twisted.internet import kqreactor
                kqreactor.install()
            elif reactor_type == 4:
                from twisted.internet import iocpreactor
                iocpreactor.install()
            from twisted.internet import reactor
            from twisted.application import internet, service
            from twisted.web import server, wsgi
        except ImportError:
            print "You need to install twisted"
            sys.exit()
        from twisted.internet import reactor
        from twisted.application import internet, service
        from twisted.web import server, wsgi

        #application = service.Application('twisted-wsgi') # call this anything you like
        resource = wsgi.WSGIResource(reactor, reactor.getThreadPool(), app)
        factory = server.Site(resource)
        reactor.listenTCP(int(port),factory)
        reactor.run()

    return action

def show_urls(app):
    def action():
        """Displays all of the url matching routes for the project."""
        for rule in app.url_map.iter_rules():
            print "%-30s"%rule, rule.endpoint
    return action


def bshell(app):
    def action():
        """run shell use bpython
        """
        from bpython import embed
        embed({"app": app})
    return action

def compile_pyc(app,use_verbose=True):
    def action(directory=('d','.'),verbose=use_verbose):
        """Compile all python files in the directory into bytecode files.
        """
        import py_compile 
        for root, dirs, files in os.walk(directory):
            for file in files:
                ext = os.path.splitext(file)[1]
                if ext==".py":
                    full_path = os.path.abspath(os.path.join(root, file))
                    if verbose:
                        print "%sc" % full_path
                    py_compile.compile(full_path)

    return action

def clean(app,use_pretend=True,use_verbose=True):
    def action(directory=('d','.'),extention=('e','.pyc'),pretend=use_pretend,verbose=use_verbose):
        """
        Clean the specify filename extention files from the directory.
        :param pretend: Instead  of  actually performing the clean,just print it
        """
        check = lambda s: s.endswith(extention)
        if pretend:
            print "The following files are supposed to be delete:"
        else:
            print "the following files are removed:"
        for dirpath, dirnames, filenames in os.walk(directory):
            for filename in filter(check, filenames):
                fullpath = os.path.abspath(os.path.join(dirpath, filename))
                if verbose:
                        print "%s" % fullpath
                if not pretend:
                    os.remove(fullpath)
    return action

def generate_secret_key(app):
    def action(length=('l',32)):
        """creates a new secret key"""
        print utils.generate_secret_key(length)
    return action


class Manager(object):
    def __init__(self, application):
        self.application = application
        self._actions = {
                'runfcgi'    :   runfcgi(application),
                'shell'     :   script.make_shell(lambda: {"app": application},
                                    "Interactive Flask Shell"),
                'runserver' :   script.make_runserver(lambda: application,
                                    use_reloader=True, threaded=True, hostname='0.0.0.0',
                                    port=7777, use_debugger=True),
                'bshell'    :   bshell(application),
                'compile_pyc' : compile_pyc(application),
                'clean'       : clean(application),
                'generate_secret_key':generate_secret_key(application),
                'runtwisted'  :   run_twisted_server(application),
                'show_urls'   :   show_urls(application),

        }

    def add_action(self, name, func):
        self._actions[name] = func(self.application)

    def run(self):
        script.run(self._actions, '')



"""
Flask-Actions
-------------

custom actions for flask

Links
`````

* `documentation <http://packages.python.org/Flask-Actions>`_
* `development version
  <http://bitbucket.org/youngking/flask-actions/get/tip.gz#egg=Flask-Actions-dev>`_


"""
from setuptools import setup
import os

def read_file(*path):
    base_dir = os.path.dirname(__file__)
    file_path = (base_dir, ) + tuple(path)
    return file(os.path.join(*file_path)).read()

setup(
    name='Flask-Actions',
    version='0.5.4',
    url='http://blog.flyzen.com',
    license='BSD',
    author='Young King',
    author_email='yanckin@gmail.com',
    description='custom actions for flask to help manage your application',
    long_description = (
        read_file("README.rst") + "\n\n" +
        "Change History\n" +
        "==============\n\n" +
        read_file("CHANGES.rst")),
    packages=['flaskext','flaskext.actions'],
    include_package_data = True,
    package_data={'flask-actions': ['flaskext/actions/project_template/*']},
    namespace_packages=['flaskext'],
    test_suite='nose.collector',
    tests_require=['Nose'],
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Werkzeug'
    ],
    scripts=['flaskext/actions/flask_admin.py'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
